@@ -1,8 +1,9 @@
 # -*- coding: utf8 -*-
 
-__version__ = '5.4.0'
+__version__ = '6.3.0'
 __all__ = [
     'OTSClient',
+    'AsyncOTSClient',
 
     # Data Types
     'INF_MIN',
@@ -39,13 +40,19 @@ __all__ = [
     'DefaultRetryPolicy',
     'LogicalOperator',
     'ComparatorType',
+    'CastType',
     'ColumnConditionType',
     'ColumnCondition',
     'CompositeColumnCondition',
+    'RegexRule',
     'SingleColumnCondition',
+    'SingleColumnRegexCondition',
     'RowExistenceExpectation',
     'SearchIndexMeta',
     'FieldSchema',
+    'VectorOptions',
+    'VectorDataType',
+    'VectorMetricType',
     'FieldType',
     'IndexSetting',
     'Collapse',
@@ -54,8 +61,10 @@ __all__ = [
     'ScoreSort',
     'GeoDistanceSort',
     'FieldSort',
+    'DocSort',
     'SortOrder',
     'SortMode',
+    'ScoreMode',
     'AnalyzerType',
     'SingleWordAnalyzerParameter',
     'SplitAnalyzerParameter',
@@ -72,6 +81,7 @@ __all__ = [
     'BoolQuery',
     'FunctionScoreQuery',
     'NestedQuery',
+    'InnerHits',
     'WildcardQuery',
     'MatchAllQuery',
     'GeoBoundingBoxQuery',
@@ -80,6 +90,14 @@ __all__ = [
     'TermsQuery',
     'SearchQuery',
     'ScanQuery',
+    'HighlightParameter',
+    'Highlight',
+    'HighlightEncoder',
+    'HighlightFragmentOrder',
+    'SearchHit',
+    'SearchInnerHit',
+    'HighlightResult',
+    'HighlightField',
     'ColumnsToGet',
     'ColumnReturnType',
     'FieldValueFactor',
@@ -89,6 +107,7 @@ __all__ = [
     'SecondaryIndexMeta',
     'SecondaryIndexType',
     'ExistsQuery',
+    'KnnVectorQuery',
     'Agg',
     'Max',
     'Min',
@@ -121,11 +140,11 @@ __all__ = [
 ]
 
 
-from tablestore.client import OTSClient
-
+from tablestore.client import OTSClient, AsyncOTSClient
 from tablestore.metadata import *
 from tablestore.aggregation import *
 from tablestore.group_by import *
 from tablestore.error import *
 from tablestore.retry import *
-from tablestore.const import *
+from tablestore.credentials import *
+from tablestore.const_module import *
